#!/usr/bin/python3
import sqlite3
import xml.dom.minidom

conn = sqlite3.connect('/tmp/tmdb.db')
conn.row_factory = sqlite3.Row

c = conn.cursor()
c.execute('CREATE INDEX IF NOT EXISTS i1 ON MOVIE_DIRECTOR(MOVIE_ID)')
c.execute('CREATE INDEX IF NOT EXISTS i2 ON DIRECTOR(ID)')
c.execute('CREATE INDEX IF NOT EXISTS i3 ON MOVIE_GENRE(MOVIE_ID)')
c.execute('CREATE INDEX IF NOT EXISTS i4 ON STAR(MOVIE_ID)')
c.execute('CREATE INDEX IF NOT EXISTS i5 ON ACTOR(ID)')
c.close()
conn.commit()


impl = xml.dom.minidom.DOMImplementation()
doctype = impl.createDocumentType(
    qualifiedName='tmdb',
    publicId='', 
    systemId='tmdb.dtd',
)    
doc = impl.createDocument('', 'tmdb', doctype)


def export_movies():

    root = doc.documentElement

    movies = conn.cursor()

    movies.execute("""SELECT ID, TITLE, RELEASE_DATE AS YEAR, LANGUAGE AS LANG 
                      FROM MOVIE""");

    for movie in movies:
        root.appendChild(export_movie(movie))

    movies.close()


def export_movie(movie):
    m = doc.createElement('movie')
    m.setAttribute('title',movie['TITLE'])
    if movie['YEAR']:
        m.setAttribute('year',movie['YEAR'])
    if movie['LANG']:
        m.setAttribute('lang',movie['LANG'])

    d = conn.cursor()
    d.execute("""SELECT D.NAME
                 FROM DIRECTOR D, MOVIE_DIRECTOR MD
                 WHERE D.ID = MD.DIRECTOR_ID AND MD.MOVIE_ID = ?
                 ORDER BY DOB DESC""",(movie['ID'],))
    director = d.fetchone()
    if director:
        m.setAttribute('director',director['NAME'])
    d.close()

    
    genres = conn.cursor()
    for genre in genres.execute("""SELECT GENRE 
                                   FROM MOVIE_GENRE 
                                   WHERE MOVIE_ID = ?""", (movie['ID'],)):
        g = doc.createElement('genre')
        g.appendChild(doc.createTextNode(genre['GENRE']))
        m.appendChild(g)
    genres.close()

    cast = conn.cursor()
    for role in cast.execute("""SELECT STAR.CHARACTER AS ROLE, ACTOR.NAME 
                                FROM STAR, ACTOR
                                WHERE STAR.ACTOR_ID = ACTOR.ID AND STAR.MOVIE_ID = ?""",(movie['ID'],)):
        c = doc.createElement('cast')
        c.setAttribute('actor',role['NAME'])
        if role['ROLE']:
            c.setAttribute('role',role['ROLE'])
        m.appendChild(c)
    cast.close()

    return m


if __name__ == '__main__':
    export_movies()
    doc.writexml(open('/tmp/tmdb-dom.xml','w'))
