#!/usr/bin/python3
import sqlite3
from xml.sax.saxutils import XMLGenerator


conn = sqlite3.connect('/tmp/tmdb.db')
conn.row_factory = sqlite3.Row

c = conn.cursor()
c.execute('CREATE INDEX IF NOT EXISTS i1 ON MOVIE_DIRECTOR(MOVIE_ID)')
c.execute('CREATE INDEX IF NOT EXISTS i2 ON DIRECTOR(ID)')
c.execute('CREATE INDEX IF NOT EXISTS i3 ON MOVIE_GENRE(MOVIE_ID)')
c.execute('CREATE INDEX IF NOT EXISTS i4 ON STAR(MOVIE_ID)')
c.execute('CREATE INDEX IF NOT EXISTS i5 ON ACTOR(ID)')
c.close()
conn.commit()


handler = XMLGenerator(open("/tmp/tmdb-sax.xml",'w'),
                       encoding='utf-8',
                       short_empty_elements=True)


def export_tmdb():
    handler.startDocument()
    handler.startElement("tmdb",{})

    export_movies()

    handler.endElement('tmdb')
    handler.endDocument()


def export_movies():

    movies = conn.cursor()

    movies.execute("""SELECT ID, TITLE, RELEASE_DATE AS YEAR, LANGUAGE AS LANG 
                      FROM MOVIE""");

    for movie in movies:
        export_movie(movie)

    movies.close()


def export_movie(movie):
    movie_attrs = {'title': movie['TITLE']}
    if movie['YEAR']:
        movie_attrs['year'] = movie['YEAR']
    if movie['LANG']:
        movie_attrs['lang'] = movie['LANG']

    d = conn.cursor()
    d.execute("""SELECT D.NAME
                 FROM DIRECTOR D, MOVIE_DIRECTOR MD
                 WHERE D.ID = MD.DIRECTOR_ID AND MD.MOVIE_ID = ?
                 ORDER BY DOB DESC""",(movie['ID'],))
    director = d.fetchone()
    if director:
        movie_attrs['director'] = director['NAME']
    d.close()

    handler.startElement("movie",movie_attrs)

    
    genres = conn.cursor()
    for genre in genres.execute("""SELECT GENRE 
                                   FROM MOVIE_GENRE 
                                   WHERE MOVIE_ID = ?""", (movie['ID'],)):
        handler.startElement("genre",{})
        handler.characters(genre['GENRE'])
        handler.endElement("genre")
    genres.close()

    cast = conn.cursor()
    for role in cast.execute("""SELECT STAR.CHARACTER AS ROLE, ACTOR.NAME 
                                FROM STAR, ACTOR
                                WHERE STAR.ACTOR_ID = ACTOR.ID AND STAR.MOVIE_ID = ?""",(movie['ID'],)):
        cast_attrs = {'actor': role['NAME']}
        if role['ROLE']:
            cast_attrs['role'] = role['ROLE']
        handler.startElement("cast",cast_attrs)
        handler.endElement("cast")
    cast.close()

    handler.endElement("movie")



if __name__ == '__main__':
    export_tmdb()

