#!/usr/bin/python3
import sqlite3

conn = sqlite3.connect('/tmp/tmdb.db')
conn.row_factory = sqlite3.Row

c = conn.cursor()
c.execute('CREATE INDEX IF NOT EXISTS i10 ON DIRECTOR(ID)')
c.execute('CREATE INDEX IF NOT EXISTS i20 ON ACTOR(ID)')
c.close()
conn.commit()

output = open('/tmp/tmdb-write-faster.xml','w')

def encode(s):
    s = s.replace('&','&amp;')
    s = s.replace('<','&lt;')
    s = s.replace('>','&gt;')
    s = s.replace('\'','&apos;')
    s = s.replace('"','&quot;')

    return s

def export_tmdb():

    output.write('<?xml version="1.0" encoding="utf-8"?>')
    output.write('<!DOCTYPE tmdb SYSTEM "tmdb.dtd">')
    output.write('<tmdb>')

    export_movies()

    output.write('</tmdb>')


def export_movies():

    movies    = conn.cursor()
    directors = conn.cursor()
    genres    = conn.cursor()
    casts     = conn.cursor()

    movies.execute("""SELECT ID, TITLE, RELEASE_DATE AS YEAR, LANGUAGE AS LANG 
                      FROM MOVIE 
                      ORDER BY ID""")

    directors.execute("""SELECT MD.MOVIE_ID, D.NAME
                         FROM DIRECTOR D, MOVIE_DIRECTOR MD 
                         WHERE D.ID = MD.DIRECTOR_ID 
                         ORDER BY MD.MOVIE_ID, D.DOB DESC""")

    genres.execute("""SELECT MOVIE_ID, GENRE 
                      FROM MOVIE_GENRE 
                      ORDER BY MOVIE_ID""")

    casts.execute("""SELECT STAR.MOVIE_ID, STAR.CHARACTER AS ROLE, ACTOR.NAME 
                     FROM STAR, ACTOR 
                     WHERE STAR.ACTOR_ID = ACTOR.ID
                     ORDER BY STAR.MOVIE_ID""")

    curr_movie = movies.fetchone()
    curr_director = directors.fetchone()
    curr_genre = genres.fetchone()
    curr_cast = casts.fetchone()

    while curr_movie:
        output.write('<movie title="%s"'%(encode(curr_movie['TITLE']),))
        if curr_movie['YEAR']:
            output.write(' year="%s"'%(curr_movie['YEAR'],))
        if curr_movie['LANG']:
            output.write(' lang="%s"'%(encode(curr_movie['LANG']),))
        
        while curr_director and curr_director['MOVIE_ID'] < curr_movie['ID']:
            curr_director = directors.fetchone()
        if curr_director and curr_director['MOVIE_ID'] == curr_movie['ID']:
            output.write(' director="%s"'%(encode(curr_director['NAME']),))

        output.write('>\n')
        
        while curr_genre and curr_genre['MOVIE_ID'] < curr_movie['ID']:
            curr_genre = genres.fetchone()
        while curr_genre and curr_genre['MOVIE_ID'] == curr_movie['ID']:
            output.write('    <genre>%s</genre>\n'%(encode(curr_genre['GENRE']),))
            curr_genre = genres.fetchone()

        while curr_cast and curr_cast['MOVIE_ID'] < curr_movie['ID']:
            curr_cast = castss.fetchone()
        while curr_cast and curr_cast['MOVIE_ID'] == curr_movie['ID']:
            output.write('    <cast actor="%s"'%(encode(curr_cast['NAME']),))
            if curr_cast['ROLE']:
                output.write(' role="%s"'%(encode(curr_cast['ROLE']),))
            output.write('/>\n')
            curr_cast = casts.fetchone()
        
        output.write('</movie>\n')
        curr_movie = movies.fetchone()

if __name__ == '__main__':
    export_tmdb()

