#!/usr/bin/python3
import sqlite3

conn = sqlite3.connect('/tmp/tmdb.db')
conn.row_factory = sqlite3.Row

c = conn.cursor()
c.execute('CREATE INDEX IF NOT EXISTS i1 ON MOVIE_DIRECTOR(MOVIE_ID)')
c.execute('CREATE INDEX IF NOT EXISTS i2 ON DIRECTOR(ID)')
c.execute('CREATE INDEX IF NOT EXISTS i3 ON MOVIE_GENRE(MOVIE_ID)')
c.execute('CREATE INDEX IF NOT EXISTS i4 ON STAR(MOVIE_ID)')
c.execute('CREATE INDEX IF NOT EXISTS i5 ON ACTOR(ID)')
c.close()
conn.commit()


movies = conn.cursor()
d = conn.cursor()
genres = conn.cursor()
cast = conn.cursor()


output = open('/tmp/tmdb-write.xml','w')

def encode(s):
    s = s.replace('&','&amp;')
    s = s.replace('<','&lt;')
    s = s.replace('>','&gt;')
    s = s.replace('\'','&apos;')
    s = s.replace('"','&quot;')

    return s


def export_tmdb():

    output.write('<?xml version="1.0" encoding="utf-8"?>')
    output.write('<!DOCTYPE tmdb SYSTEM "tmdb.dtd">')
    output.write('<tmdb>')

    export_movies()

    output.write('</tmdb>')



def export_movies():

    movies.execute("""SELECT ID, TITLE, RELEASE_DATE AS YEAR, LANGUAGE AS LANG 
                      FROM MOVIE""");

    for movie in movies:
        export_movie(movie)


def export_movie(movie):
    output.write('<movie title="%s"'%(encode(movie['TITLE']),))
    if movie['YEAR']:
        output.write(' year="%s"'%(movie['YEAR'],))
    if movie['LANG']:
        output.write(' lang="%s"'%(encode(movie['LANG']),))
    d.execute("""SELECT D.NAME
                 FROM DIRECTOR D, MOVIE_DIRECTOR MD
                 WHERE D.ID = MD.DIRECTOR_ID AND MD.MOVIE_ID = ?
                 ORDER BY DOB DESC""",(movie['ID'],))
    director = d.fetchone()
    if director:
        output.write(' director="%s"'%(encode(director['NAME']),))
    output.write('>\n')
    
    for genre in genres.execute("""SELECT GENRE 
                                   FROM MOVIE_GENRE 
                                   WHERE MOVIE_ID = ?""", (movie['ID'],)):
        output.write('   <genre>%s</genre>\n'%(encode(genre['GENRE']),))

    for role in cast.execute("""SELECT STAR.CHARACTER AS ROLE, ACTOR.NAME 
                                FROM STAR, ACTOR
                                WHERE STAR.ACTOR_ID = ACTOR.ID AND STAR.MOVIE_ID = ?""",(movie['ID'],)):
        output.write('   <cast actor="%s"'%(encode(role['NAME']),))
        if role['ROLE']:
            output.write(' role="%s"'%(encode(role['ROLE']),))
        output.write('/>\n')

    output.write('</movie>\n')



if __name__ == '__main__':
    export_tmdb()

